"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("@babel/types"));
function nameForReactComponent(path) {
    const { parentPath } = path;
    if (!t.isArrowFunctionExpression(path.node) && t.isIdentifier(path.node.id)) {
        return path.node.id;
    }
    if (t.isVariableDeclarator(parentPath)) {
        // @ts-ignore
        return parentPath.node.id;
    }
    return null;
}
const DEFAULT_DATA_TESTID = 'data-testid';
function createDataAttribute(name) {
    return t.jsxAttribute(t.jsxIdentifier(DEFAULT_DATA_TESTID), t.stringLiteral(name));
}
function hasDataAttribute(node) {
    return node.attributes.some(attribute => t.isJSXAttribute(attribute) &&
        t.isJSXIdentifier(attribute.name, { name: DEFAULT_DATA_TESTID }));
}
const returnStatementVistor = {
    // topがフラグメントのときはスキップする
    JSXFragment(path) {
        path.skip();
    },
    JSXElement(path, { name }) {
        const openingElement = path.get('openingElement');
        // topにあるJSX Elementのみ処理する
        path.skip();
        // すでにdata-testidがある場合は処理しない
        if (hasDataAttribute(openingElement.node)) {
            return;
        }
        openingElement.node.attributes.push(createDataAttribute(name));
    },
};
const functionVisitor = {
    ReturnStatement(path, state) {
        const arg = path.get('argument');
        if (!arg.isIdentifier()) {
            path.traverse(returnStatementVistor, state);
        }
    },
};
function plugin() {
    return {
        name: 'react-data-testid',
        visitor: {
            'FunctionExpression|ArrowFunctionExpression|FunctionDeclaration': (path) => {
                const identifier = nameForReactComponent(path);
                if (!identifier) {
                    return;
                }
                if (path.isArrowFunctionExpression()) {
                    path.traverse(returnStatementVistor, { name: identifier.name });
                }
                else {
                    path.traverse(functionVisitor, { name: identifier.name });
                }
            },
        },
    };
}
exports.default = plugin;
//# sourceMappingURL=index.js.map